CASSHI ; #######################################################
 ; HIGH SPEED TAPE LOAD AND SAVE ROUTINE OPERATING AT ....
 ; BAUD.  THIS CAN BE MADE TRANSPARENT TO BASIC BY USING
 ; EITHER /SAVE AND /LOAD OR /PUT AND /GET.  THE ROUTINE
 ; PRESENTED HERE SAVES A BLOCK OF MACHINE CODE, BUT MAY
 ; BE USED FOR SAVING AND LOADING BASIC WITH MINOR CHANGES
 ; #######################################################
 ;
 	ORG	7000H		; TOP OF MEMORY - RELOC.
 ;
 ; #######################################################
 ; WRITE-BYTE SUBROUTINE.  BYTE IS ASSUMED IN C REGISTER
 ; #######################################################
 ;
 WRITE	LD	B,8		; NUMBER OF BITS TO WRITE
 	PUSH	BC		; SAVE BITS OF BYTE
 	LD	B,10H		; GET A DELAY VALUE
 	DJNZ	$		; AND DELAY 117 USECS
 	POP	BC		; AND RESTORE BITS
 WLOOP	CALL	TIMEX		; WRITE TIMING BIT
 	RL	C		; ROTATE BIT INTO FLAG
 	RLA			; ROTATE FLAG INTO A
 	CALL	BITEX		; WRITE BIT TO TAPE
 	DJNZ	WLOOP		; DO IT TOTAL OF 8 TIMES
 	RET			; BACK FROM WRITE ROUTINE
 ;
 ; #######################################################
 ; ROUTINE TO WRITE TIMING BIT POSITIVE-NEGATIVE TO TAPE
 ; #######################################################
 ;
 TIMEX	PUSH	BC		; SAVE B ON STACK
 	LD	A,(403DH)	; GET SCREEN INFORMATION
 	AND	0FCH		; MASK OUT LOW TWO BITS
 	OR	1		; SET LOWEST BIT
 	OUT	(0FFH),A	; WRITE BIT TO TAPE
 	LD	B,07H		; TIMING VALUE @ 1.77 MHZ
 	DJNZ	$		; IS EQUAL TO 67 USECS
 	AND	0FCH		; MASK OUT LOW TWO BITS
 	OR	2		; SET NEXT LOWEST BIT
 	OUT	(0FFH),A	; WRITE NEG. BIT TO TAPE
 	LD	B,07H		; TIMING VALUE @ 1.77 MHZ
 	DJNZ	$		; IS EQUAL TO 64 USECS
 	AND	0FCH		; MASK OUT LOW TWO BITS
 	OUT	(0FFH),A	; WRITE NEUTRAL BIT 
 	LD	B,07H		; TIMING VALUE @ 1.77 MHZ
 	DJNZ	$		; IS EQUAL TO 113 USECS
 	POP	BC		; RESTORE NUMBER OF BITS
 	RET			; BACK TO BIT WRITE ROUT.
 ;
 ; #######################################################
 ; ROUTINE TO WRITE INDIVIDUAL BIT TO TAPE (OR NOT IF 0)
 ; #######################################################
 ;
 BITEX	PUSH	BC		; SAVE NUMBER OF BITS
 	AND	1		; MASK OUT ALL BUT BIT 0
 	LD	B,A		; SAVE A IN B REGISTER
 	LD	A,(403DH)	; GET SCREEN STATUS
 	AND	0FCH		; MASK OUT LOW TWO BITS
 	ADD	A,B		; SET BIT OR NOT
 	OUT	(0FFH),A	; SEND OUT CASSETTE PORT
 	PUSH	BC		; SAVE B REGISTER AGAIN
 	LD	B,05H		; TIMING VALUE FOR BIT
 	DJNZ	$		; DELAY IS 68 USECS
 	POP	BC		; GET VALUE BACK INTO B
 	RLC	B		; ROTATE INTO BIT 1
 	AND	0FCH		; MASK OUT LOW BITS AGAIN
 	ADD	A,B		; SET BIT 1 OR NOT
 	OUT	(0FFH),A	; SEND OUT CASSETTE PORT
 	PUSH	BC		; SAVE B REGISTER AGAIN
 	LD	B,06H		; DELAY IS 63 USECS
 	DJNZ	$		; DELAY FOR BOTTOM OF BIT
 	AND	0FCH		; CREATE A NEUTRAL BIT
 	OUT	(0FFH),A	; AND WRITE IT TO TAPE
 	LD	B,08H		; GET DELAY VALUE IN B
 	DJNZ	$		; DELAY IS 110 USECS
 	POP	BC		; CLEAR STACK OF BIT
 	POP	BC		; GET ORIGINAL BITS BACK
 	RET			; BACK TO SAVING ROUTINE
 ;
 ; #######################################################
 ; ROUTINE TO WRITE LEADER OF 256 BYTES OF FF PLUS SYNC A5
 ; #######################################################
 ;
 SYNCHR	LD	B,0		; 100 HEX = 256 BYTES
 SYLOOP	PUSH	BC		; SAVE THIS VALUE
 	LD	C,0		; WRITE BYTE OF 1111 1111
 	CALL	WRITE		; WRITE THAT BYTE
 	POP	BC		; RESTORE VALUE FOR USE
 	DJNZ	SYLOOP		; WRITE TOTAL OF FF BYTES
 	LD	C,0A5H		; SYNCHRONIZATION BYTE
 	CALL	WRITE		; WRITE THAT BYTE
 	RET			; BACK TO MAIN ROUTINE
 ;
 ; #######################################################
 ;
 ; .......................................................
 ;TEMPORARY ROUTINE TO GET MATERIAL FROM MEMORY AND WRITE
 ;
 TEMPEX	DI			; GET RID OF THE BOTHER
 	LD	HL,0000		; START AT ROM BEGINNING
  	CALL	SYNCHR		; WRITE LEADER AND SYNC
 LOOOOP	INC	HL		; GET NEXT MEMORY BYTE
 	LD	A,(HL)		; GET VALUE AT MEM LOC'N
 	LD	C,A		; PUT BYTE IN C REGISTER
 	CALL	WRITE		; WRITE BYTE TO TAPE
 	JP	LOOOOP		; LOOP AROUND FOREVER
 ; .......................................................
 ;
 	END	TEMPEX
