CASSRE ; #######################################################
 ; HIGH SPEED TAPE LOAD AND SAVE ROUTINE OPERATING AT ....
 ; BAUD.  THIS CAN BE MADE TRANSPARENT TO BASIC BY USING
 ; EITHER /SAVE AND /LOAD OR /PUT AND /GET.  THE ROUTINE
 ; PRESENTED HERE SAVES A BLOCK OF MACHINE CODE, BUT MAY
 ; BE USED FOR SAVING AND LOADING BASIC WITH MINOR CHANGES
 ; #######################################################
 ;
 	ORG	7000H		; TOP OF MEMORY - RELOC.
 ;
 ;
 ; #######################################################
 ; READ BYTE SUBROUTINE: COMPLETED BYTE INTO C REGISTER
 ; #######################################################
 ;
 READ	LD	B,8		; NUMBER OF BITS TO READ
 RLOOP	CALL	REDEX		; READ TIMING BIT
 	CALL	REDBIT		; READ DATA BIT IF ANY
 	RLA			; ROTATE BIT INTO CARRY
 	RL	C		; ROTATE CARRY INTO C
 	DJNZ	RLOOP		; DO IT TOTAL OF 8 TIMES
 	RET			; BACK FROM READ ROUTINE
 ;
 ; #######################################################
 ; ROUTINE TO READ TIMING BIT FROM TAPE AND DELAY AFTER
 ; #######################################################
 ;
 REDEX	CALL	INSIG		; KEEP LOOKING FOR ONE
 REDX2	IN	A,(0FFH)	; KEEP LOOKING FOR ONE
 	RLA			; ROTATE INTO CARRY
 	JR	NC,REDX2	; KEEP LOOKING FOR ONE
 	PUSH	BC		; SAVE NUMBER OF BITS
 	LD	B,0FH		; VALUE TO DELAY
 	DJNZ	$		; AND DELAY FOR 144 USECS
 	POP	BC		; RESTORE NUMBER OF BITS
 	CALL	INSIG		; RESET INSIG FLIP-FLOP
 	RET			; BACK TO MAIN ROUTINE
 ;
 ; #######################################################
 ; ROUTINE TO READ INDIVIDUAL DATA BIT (IF ANY) FROM TAPE
 ; #######################################################
 ;
 REDBIT	PUSH	BC		; SAVE VALUE IN B AND C
 	LD	B,18H		; VALUE FOR DELAY
 	DJNZ	$		; EQUAL TO 197 USECS
 	IN	A,(0FFH)	; GET VALUE FROM CASSETTE
 	POP	BC		; RESTORE VALUES TO B & C
 	RET			; RETURN WITH VALUE IN A
 ;
 ; #######################################################
 ; THIS ROUTINE READS THROUGH THE LEADER FOR SYNC BYTE
 ; #######################################################
 ;
 REDSYN	CALL	REDEX		; READ TIMING BIT
 	CALL	REDBIT		; GET BIT FROM TAPE
 	RLA			; ROTATE INTO CARRY FLAG
 	JR	NC,REDSYN	; KEEP LOOKING FOR A 1
 	RL	C		; ROTATE INTO C REGISTER
 	LD	B,7		; NUMBER OF BITS LEFT
 SYNCLP	CALL	REDEX		; READ TIMING BIT
 	CALL	REDBIT		; GET BIT FROM TAPE
 	RLA			; ROTATE INTO CARRY FLAG
 	RL	C		; ROTATE INTO C REGISTER
 	DJNZ	SYNCLP		; DO IT 7 TIMES LEFT
 	LD	A,0A5H		; LOAD SYNC BYTE VALUE
 	CP	C		; COMPARE AGAINST C
 	RET	Z		; RETURN IF A MATCH
 	JR	REDSYN		; BACK TO KEEP LOOKING
 ;
 ; #######################################################
 ; ROUTINE TO RESET INSIG FLIP-FLOP TO BE ABLE TO READ
 ; #######################################################
 ;
 INSIG	PUSH	AF		; SAVE VALUE IN AF
 	LD	A,(403DH)	; GET SCREEN STATUS BYTE
 	OUT	(0FFH),A	; RESET FLIP-FLOP
 	POP	AF		; RESTORE AF STATUS
 	RET			; BACK TO READING ROUTINE
 ;
 ;
 ;
 ; TEMPORARY ROUTINE TO READ BYTES FROM TAPE
 ;
 TEMPER	DI			; GET RID OF BOTHERS
 	CALL	REDSYN		; CALL READ SYNC ROUTINE
 	LD	HL,3C00H	; GET VIDEO ADDRESS
 TREAD	CALL	READ		; READ ONE BYTE
 	LD	(HL),C		; GET VALUE TO VIDEO MEM
 	INC	HL		; INCREMENT MEMORY LOC'N
 	LD	A,H		; GET VALUE OF VIDEO MSB
 	CP	40H		; IS IT PAST VIDEO YET?
 	JR	NZ,TREAD	; CONTINUE IF OKAY
 	LD	H,3CH		; MAKE BEGINNING OF VIDEO
 	JR	TREAD		; AND THEN GO BACK
 ;
 ;
 ;	END	TEMPEX
 	END	TEMPER
