HIGHBA ; #######################################################
 ; HIGH SPEED TAPE LOAD AND SAVE ROUTINE OPERATING AT 1960
 ; BAUD.  THIS CAN BE MADE TRANSPARENT TO BASIC BY USING
 ; EITHER /SAVE AND /LOAD OR /PUT AND /GET.  THE ROUTINE
 ; PRESENTED HERE SAVES A BLOCK OF MACHINE CODE, BUT MAY
 ; BE USED FOR SAVING AND LOADING BASIC BY LOCATING START
 ; AND END OF THE BASIC PROGRAM (FOR WRITE), AND START AND
 ; END OF BASIC MEMORY (FOR READ).  SAMPLE ACCESS ROUTINES
 ; ARE PRESENTED AT THE END OF THIS LISTING.
 ; #######################################################
 ;
 	ORG	7000H		; TOP OF MEMORY - RELOC.
 ;
 ; #######################################################
 ; WRITE-BYTE SUBROUTINE.  BYTE IS ASSUMED IN C REGISTER
 ; #######################################################
 ;
 WRITE	LD	B,8		; NUMBER OF BITS TO WRITE
 	PUSH	BC		; SAVE BITS OF BYTE
 	LD	B,1EH		; GET A DELAY VALUE
 	DJNZ	$		; AND DELAY 117 USECS
 	POP	BC		; AND RESTORE BITS
 WLOOP	CALL	TIMEX		; WRITE TIMING BIT
 	RL	C		; ROTATE BIT INTO FLAG
 	RLA			; ROTATE FLAG INTO A
 	CALL	BITEX		; WRITE BIT TO TAPE
 	DJNZ	WLOOP		; DO IT TOTAL OF 8 TIMES
 	RET			; BACK FROM WRITE ROUTINE
 ;
 ; #######################################################
 ; ROUTINE TO WRITE TIMING BIT POSITIVE-NEGATIVE TO TAPE
 ; #######################################################
 ;
 TIMEX	PUSH	BC		; SAVE B ON STACK
 	LD	A,(403DH)	; GET SCREEN INFORMATION
 	AND	0FCH		; MASK OUT LOW TWO BITS
 	OR	1		; SET LOWEST BIT
 	OUT	(0FFH),A	; WRITE BIT TO TAPE
 	LD	B,07H		; TIMING VALUE @ 1.77 MHZ
 	DJNZ	$		; IS EQUAL TO 67 USECS
 	AND	0FCH		; MASK OUT LOW TWO BITS
 	OR	2		; SET NEXT LOWEST BIT
 	OUT	(0FFH),A	; WRITE NEG. BIT TO TAPE
 	LD	B,07H		; TIMING VALUE @ 1.77 MHZ
 	DJNZ	$		; IS EQUAL TO 64 USECS
 	AND	0FCH		; MASK OUT LOW TWO BITS
 	OUT	(0FFH),A	; WRITE NEUTRAL BIT 
 	LD	B,07H		; TIMING VALUE @ 1.77 MHZ
 	DJNZ	$		; IS EQUAL TO 113 USECS
 	POP	BC		; RESTORE NUMBER OF BITS
 	RET			; BACK TO BIT WRITE ROUT.
 ;
 ; #######################################################
 ; ROUTINE TO WRITE INDIVIDUAL BIT TO TAPE (OR NOT IF 0)
 ; #######################################################
 ;
 BITEX	PUSH	BC		; SAVE NUMBER OF BITS
 	AND	1		; MASK OUT ALL BUT BIT 0
 	LD	B,A		; SAVE A IN B REGISTER
 	LD	A,(403DH)	; GET SCREEN STATUS
 	AND	0FCH		; MASK OUT LOW TWO BITS
 	ADD	A,B		; SET BIT OR NOT
 	OUT	(0FFH),A	; SEND OUT CASSETTE PORT
 	PUSH	BC		; SAVE B REGISTER AGAIN
 	LD	B,05H		; TIMING VALUE FOR BIT
 	DJNZ	$		; DELAY IS 68 USECS
 	POP	BC		; GET VALUE BACK INTO B
 	RLC	B		; ROTATE INTO BIT 1
 	AND	0FCH		; MASK OUT LOW BITS AGAIN
 	ADD	A,B		; SET BIT 1 OR NOT
 	OUT	(0FFH),A	; SEND OUT CASSETTE PORT
 	PUSH	BC		; SAVE B REGISTER AGAIN
 	LD	B,06H		; DELAY IS 63 USECS
 	DJNZ	$		; DELAY FOR BOTTOM OF BIT
 	AND	0FCH		; CREATE A NEUTRAL BIT
 	OUT	(0FFH),A	; AND WRITE IT TO TAPE
 	LD	B,08H		; GET DELAY VALUE IN B
 	DJNZ	$		; DELAY IS 110 USECS
 	POP	BC		; CLEAR STACK OF BIT
 	POP	BC		; GET ORIGINAL BITS BACK
 	RET			; BACK TO SAVING ROUTINE
 ;
 ; #######################################################
 ; ROUTINE TO WRITE LEADER OF 256 BYTES OF FF PLUS SYNC A5
 ; #######################################################
 ;
 SYNCHR	LD	B,0		; 100 HEX = 256 BYTES
 SYLOOP	PUSH	BC		; SAVE THIS VALUE
 	LD	C,0		; WRITE BYTE OF 1111 1111
 	CALL	WRITE		; WRITE THAT BYTE
 	LD	B,8		; SHORT LOOP BETW. BYTES
 	DJNZ	$		; DELAY JUST 58 USECS
 	POP	BC		; RESTORE VALUE FOR USE
 	DJNZ	SYLOOP		; WRITE TOTAL OF FF BYTES
 	LD	C,0A5H		; SYNCHRONIZATION BYTE
 	CALL	WRITE		; WRITE THAT BYTE
 	RET			; BACK TO MAIN ROUTINE
 ;
 ; #######################################################
 ; READ BYTE SUBROUTINE: COMPLETED BYTE INTO C REGISTER
 ; #######################################################
 ;
 READ	LD	B,8		; NUMBER OF BITS TO READ
 RLOOP	CALL	REDEX		; READ TIMING BIT
 	CALL	REDBIT		; READ DATA BIT IF ANY
 	RLA			; ROTATE BIT INTO CARRY
 	RL	C		; ROTATE CARRY INTO C
 	DJNZ	RLOOP		; DO IT TOTAL OF 8 TIMES
 	RET			; BACK FROM READ ROUTINE
 ;
 ; #######################################################
 ; ROUTINE TO READ TIMING BIT FROM TAPE AND DELAY AFTER
 ; #######################################################
 ;
 REDEX	CALL	INSIG		; KEEP LOOKING FOR ONE
 REDX2	IN	A,(0FFH)	; KEEP LOOKING FOR ONE
 	RLA			; ROTATE INTO CARRY
 	JR	NC,REDX2	; KEEP LOOKING FOR ONE
 	PUSH	BC		; SAVE NUMBER OF BITS
 	LD	B,0FH		; VALUE TO DELAY
 	DJNZ	$		; AND DELAY FOR 144 USECS
 	POP	BC		; RESTORE NUMBER OF BITS
 	CALL	INSIG		; RESET INSIG FLIP-FLOP
 	RET			; BACK TO MAIN ROUTINE
 ;
 ; #######################################################
 ; ROUTINE TO READ INDIVIDUAL DATA BIT (IF ANY) FROM TAPE
 ; #######################################################
 ;
 REDBIT	PUSH	BC		; SAVE VALUE IN B AND C
 	LD	B,18H		; VALUE FOR DELAY
 	DJNZ	$		; EQUAL TO 197 USECS
 	IN	A,(0FFH)	; GET VALUE FROM CASSETTE
 	POP	BC		; RESTORE VALUES TO B & C
 	RET			; RETURN WITH VALUE IN A
 ;
 ; #######################################################
 ; THIS ROUTINE READS THROUGH THE LEADER FOR SYNC BYTE
 ; #######################################################
 ;
 REDSYN	CALL	REDEX		; READ TIMING BIT
 	CALL	REDBIT		; GET BIT FROM TAPE
 	RLA			; ROTATE INTO CARRY FLAG
 	JR	NC,REDSYN	; KEEP LOOKING FOR A 1
 	RL	C		; ROTATE INTO C REGISTER
 	LD	B,7		; NUMBER OF BITS LEFT
 SYNCLP	CALL	REDEX		; READ TIMING BIT
 	CALL	REDBIT		; GET BIT FROM TAPE
 	RLA			; ROTATE INTO CARRY FLAG
 	RL	C		; ROTATE INTO C REGISTER
 	DJNZ	SYNCLP		; DO IT 7 TIMES LEFT
 	LD	A,0A5H		; LOAD SYNC BYTE VALUE
 	CP	C		; COMPARE AGAINST C
 	RET	Z		; RETURN IF A MATCH
 	JR	REDSYN		; BACK TO KEEP LOOKING
 ;
 ; #######################################################
 ; ROUTINE TO RESET INSIG FLIP-FLOP TO BE ABLE TO READ
 ; #######################################################
 ;
 INSIG	PUSH	AF		; SAVE VALUE IN AF
 	LD	A,(403DH)	; GET SCREEN STATUS BYTE
 	OUT	(0FFH),A	; RESET FLIP-FLOP
 	POP	AF		; RESTORE AF STATUS
 	RET			; BACK TO READING ROUTINE
 ;
 ; #######################################################
 ; THIS ROUTINE WILL LOAD A BASIC PROGRAM WITH NO FILENAME
 ; #######################################################
 ;
 TEMPER	DI			; GET RID OF BOTHERS
 	LD	HL,(40B1H)	; GET TOP OF BASIC MEMORY
 	LD	DE,(40A4H)	; GET BASIC PROGRAM PTR.
 	PUSH	DE		; SAVE IT A MOMENT
 	SBC	HL,DE		; GET MEMORY AVAILABLE
 	PUSH	HL		; GET READY FOR TRANSFER
 	POP	BC		; COUNT OF MEMORY IS IN B
 	POP	HL		; BEGINNING OF BASIC = HL
 	PUSH	BC		; SAVE AVAILABLE MEMORY
 	CALL	REDSYN		; CALL READ SYNC ROUTINE
 	POP	BC		; RESTORE AVAILABLE MEM.
 TREAD	PUSH	BC		; SAVE BYTE COUNT
 	CALL	READ		; READ ONE BYTE
 	LD	(HL),C		; GET VALUE TO VIDEO MEM
 	POP	BC		; GET BYTE COUNT BACK
 	DEC	BC		; REDUCE COUNT BY ONE
 	LD	A,B		; GET HIGH BYTE OF MEMORY
 	OR	C		; AND CHECK AGAINST LOW
 	JP	Z,1997H		; OM ERROR IF TOO MUCH
 	LD	A,(HL)		; GET VALUE IN HL
 	AND	A		; TEST IF A ZERO
 	JP	NZ,JUMP3	; PAST FLASH IF OKAY
 	LD	A,(3C3FH)	; GET PLACE ON SCREEN
 	XOR	0AH		; AND TOGGLE STAR & SPACE
 	LD	(3C3FH),A	; AND PUT BACK ON SCREEN
 	DEC	HL		; GO BACK SPACE FOR TEST
 	LD	A,(HL)		; GET VALUE THERE
 	AND	A		; TEST IF A ZERO ALSO
 	JP	Z,JUMP4		; GO TO END ROUTINE IF 0
 	INC	HL		; BACK TO PROPER BYTE
 JUMP3	INC	HL		; READY NEXT MEM LOC'N
 	JR	TREAD		; AND THEN GO BACK
 JUMP4	LD	DE,(40A4H)	; GET START OF PRGRM PTR
 	LD	A,0FFH		; GET RESETTING CODE
 	LD	(DE),A		; PUT AT PROGRAM START
 	CALL	1AFCH		; RESET ALL LINE NUMBERS
 	INC	HL		; HL MOVED PAST PROGRAM
 	LD	(40F9H),HL	; SIMPLE VARIABLE POINTER
 	CALL	01C9H		; CLEAR THE SCREEN NOW
 	CALL	1B61H		; CLEAR ALL THE POINTERS
 	JP	06CCH		; GO TO BASIC "READY"
 ;
 ; #######################################################
 ; THIS ROUTINE IS A GERMINAL ROUTINE TO CSAVE A PROGRAM
 ; WITHOUT A PROGRAM NAME.  FORMAT:  /PUT.  THE 500-BAUD
 ; BLOCK SAVE CAN BE USED AS AN EXAMPLE OF HOW TO EMPLOY
 ; A PROGRAM NAME IN SAVING A PROGRAM.
 ; #######################################################
 ;
 TEMPEX	DI			; GET RID OF BOTHER
 	LD	A,2AH		; GET READY AN ASTERISK
 	LD	(3C3EH),A	; PLACE STAR ON SCREEN
 	LD	(3C3FH),A	; PLACE STAR NEXT TO IT
 	LD	HL,(40A4H)	; START OF BASIC PROGRAM
 	LD	DE,(40F9H)	; BOTTOM OF VAR. POINTER
 	CALL	SYNCHR		; WRITE LEADER AND SYNC
 LOOOOP	LD	A,H		; GET CURRENT HIGH MSB
 	CP	D		; SAME AS TARGET MSB?
 	JP	NZ,JUMP01	; CONTINUE IF NOT SAME
 	LD	A,L		; SAME HI MSB - READY LOW
 	CP	E		; SAME AS TARGET LSB?
 	RET	Z		; DONE WITH SAVE IF SO
 JUMP01	LD	A,(HL)		; ELSE GET VALUE IN MEM
 	AND	A		; TEST IF A ZERO
 	JP	NZ,JUMP02	; JUST GO ON IF NOT
 	PUSH	AF		; SAVE VALUE IN A
 	LD	A,(3C3FH)	; GET TOGGLE VALUE TO A
 	XOR	0AH		; TOGGLE STAR & SPACE
 	LD	(3C3FH),A	; AND PUT IT ON SCREEN
 	POP	AF		; GET VALUE BACK TO A
 JUMP02	LD	C,A		; PUT BYTE IN C REGISTER
 	CALL	WRITE		; WRITE BYTE TO TAPE
 	INC	HL		; GET NEXT MEMORY LOC'N
 	JP	LOOOOP		; LOOP AROUND FOREVER
 ;
 ; #######################################################
 ; REMEMBER:  ENTRY POINT FOR SAVE ROUTINE IS  TEMPEX
 ; REMEMBER:  ENTRY POINT FOR LOAD ROUTINE IS  TEMPER
 ; #######################################################
 ;
 	END
 ;
